
# Programmer: Jeremiah Dodds
# Email:      jeremiah.dodds@gmail.com
#
# Copyright 2008 Jeremiah Dodds
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Simple webapp for providing the service of downloading a package and its 
dependencies from the repositories"""

import os
import subprocess
import tarfile
import cherrypy
from cherrypy.lib import static

class PackageList(object):
    """A list of packages (or at least their names) available for download.
    At this point in time, this does a rather simplistic parsing through the
    package resource files used by apt (these normally reside in /var/lib/apt/lists)"""

    #if your apt config has a non-default option for Dir::State::Lists, you'll
    #need to change this. you probably don't need to. 
    packages_dir = "/var/lib/apt/lists/"

    def __init__(self):
        #there are occasionally packages available from different repos.
        #instead of trying to be robust, we're just discarding duplicates, as 
        #our needs are simple at this point in time.
        self.packages = set() 
        self.__parse()

    def __parse(self):
        """Parse through the files in packages_dir to pull their names out"""
        name_trigger = "Package:"
        file_paths = [os.path.join(self.packages_dir, x)
                      for x in os.listdir(self.packages_dir) 
                      if x.endswith("Packages")]  #files containing binary package information end in "Packages"
        for path in file_paths:
            afile = open(path, "r")
            #whoo for really simplistic parsing...
            for line in afile:
                if line.startswith(name_trigger):
                    self.packages.add(line.split(name_trigger)[1].strip())

    def search(self, string):
        """Search for packages that start with string"""
        results = [s for s in self.packages if s.startswith(string)]
        results.sort()
        return results
    
class PackageDepends(object):
    """Provides a web-interface for downloading packages and their dependencies"""
    package_list = PackageList()

    #presumably this website's root
    basedir = os.getcwd()
    #directory to create / serve the generated tarfiles
    working_dir = os.path.join(basedir, 'data', "packages")

    def __generate_package(self, package):
        """ Downloads a package and it's buld-dependencies, creates a tar.gz
        archive of them, deletes everything but the tar.gz, and returns the path
        of the generated tarfile"""
        
        packpath = os.path.join(self.working_dir, package)
        archive_name = os.path.join(packpath, package + ".tar.gz")
        
        #if we've already had a request for this, the directory for it should exist
        #and we should already have a tarfile for it
        if not os.path.isdir(packpath):
            #make sure we preserve where we are
            olddir = os.getcwd()

            os.mkdir(packpath)
            os.chdir(packpath)
            
            #apt-get build-dep needs this directory to work in
            os.mkdir("partial")
            
            #download the package and it's dependencies
            subprocess.call(["aptitude", "download", package])
            subprocess.call(["sudo", "apt-get", "build-dep", "-dy", "-oDir::Cache::Archives=%s"%(packpath), package])
            
            note = open("README.txt", "w")
            note.write("To install all the debs here, \"sudo dpkg -i *deb\" should work.\n")
            note.close()
            
            #now we generate our tar.gz file out of all the debs we just downloaded
            archive = tarfile.open(archive_name, "w:gz")
            for afile in [f for f in os.listdir('.') if f.endswith(".deb")]:
                archive.add(afile)
            archive.add("README.txt")
            archive.close()
            
            #cleanup
            os.rmdir("partial")
            for afile in [f for f in os.listdir('.') if not f.endswith(".tar.gz")]:
                os.remove(afile)
            
            #and get back to where we were, just in case
            os.chdir(olddir)
        return archive_name
    
    def index(self):
        """Incredibly simplistic at this point in time, just a search form"""
        f = open("data/templates/main.html", 'r')
        return f.read()
    index.exposed = True

    def list_results(self, package=None):
        """Generates a list of links to download packages that met the search query"""
        if package:
            matches = self.package_list.search(package)
            #this is only here because I'm trying out yield statements instead of
            #lots of string concatenation... I kinda like it
            yield"<h3>Click a name to download it + it's dependencies</h3>"
            for match in matches:
                yield "<a href='download?package=" + match + "'>" + match + "</a><br/>"

    list_results.exposed = True
    
    def download(self, package=None):
        """Sends a zipped tar of the package and its dependencies"""
        if package:
            filepath = self.__generate_package(package)
            return static.serve_file(filepath, "application/x-gzip",
                                     "attachment", os.path.basename(filepath))
                                     
    download.exposed = True
    
#yeah, nothing complex going on here right now.
if __name__ == '__main__':
    current_dir = os.path.dirname(os.path.abspath(__file__))
    conf = {'/': {'tools.staticdir.root': os.path.join(current_dir, 'data')},
            '/js': {'tools.staticdir.on': True,
                    'tools.staticdir.dir': 'js'}}
    cherrypy.quickstart(PackageDepends(), '/', config=conf)
    
